@@ -2,7 +2,8 @@ from typing import Any
 
 from ffun.librarian import errors
 from ffun.library.entities import Entry
-from ffun.ontology.entities import NormalizationMode, RawTag
+from ffun.ontology.entities import RawTag
+from ffun.tags.entities import TagCategory
 
 
 class Processor:
@@ -32,7 +33,7 @@ class AlwaysConstantProcessor(Processor):
         self._tags = tags
 
     async def process(self, entry: Entry) -> list[RawTag]:
-        return [RawTag(raw_uid=tag, normalization=NormalizationMode.final) for tag in self._tags]
+        return [RawTag(raw_uid=tag, categories={TagCategory.test_final}) for tag in self._tags]
 
 
 class AlwaysSkipEntryProcessor(Processor):
