@@ -1,60 +1,38 @@
 from collections import Counter
 from typing import Iterable
 
-from bidict import bidict
-
 from ffun.core.postgresql import ExecuteType, execute, run_in_transaction, transaction
 from ffun.domain.entities import EntryId, TagId, TagUid
-from ffun.ontology import operations
-from ffun.ontology.entities import NormalizedTag, Tag, TagCategory, TagPropertyType
+from ffun.ontology import cache, operations
+from ffun.ontology.entities import NormalizedTag, Tag, TagProperty, TagPropertyType
 from ffun.tags import converters
+from ffun.tags.entities import TagCategory
 
 count_total_tags = operations.count_total_tags
 count_total_tags_per_type = operations.count_total_tags_per_type
 count_total_tags_per_category = operations.count_total_tags_per_category
 count_new_tags_at = operations.count_new_tags_at
 tag_frequency_statistics = operations.tag_frequency_statistics
+get_tags_properties = operations.get_tags_properties
 
 
-_tags_cache: bidict[TagUid, TagId] = bidict()
-
-
-async def get_id_by_uid(tag: TagUid) -> TagId:
-    if tag in _tags_cache:
-        return _tags_cache[tag]
-
-    tag_id = await operations.get_or_create_id_by_tag(tag)
-
-    _tags_cache[tag] = tag_id
-
-    return tag_id
+_tags_cache = cache.TagsCache()
 
 
 async def get_ids_by_uids(tags: Iterable[TagUid]) -> dict[TagUid, TagId]:
-    return {tag: await get_id_by_uid(tag) for tag in tags}
+    return await _tags_cache.ids_by_uids(tags)
 
 
 async def get_tags_by_ids(ids: Iterable[TagId]) -> dict[TagId, TagUid]:
-    result = {}
+    return await _tags_cache.uids_by_ids(ids)
 
-    tags_to_request = []
 
-    for tag_id in ids:
-        if tag_id in _tags_cache.inverse:
-            result[tag_id] = _tags_cache.inverse[tag_id]
-        else:
-            tags_to_request.append(tag_id)
+async def apply_tags_properties(properties: list[TagProperty]) -> None:
+    await operations.apply_tags_properties(execute, properties)
 
-    if not tags_to_request:
-        return result
 
-    missed_tags = await operations.get_tags_by_ids(tags_to_request)
-
-    _tags_cache.inverse.update(missed_tags)
-
-    result.update(missed_tags)
-
-    return result
+async def copy_tag_properties(processor_id: int, old_tag_id: TagId, new_tag_id: TagId) -> None:
+    await operations.copy_tag_properties(execute, processor_id, old_tag_id, new_tag_id)
 
 
 # TODO: in the future we could split this function into two separate functions
@@ -86,7 +64,7 @@ async def apply_tags_to_entry(entry_id: EntryId, processor_id: int, tags: Iterab
         properties.extend(tag.build_properties_for(tag_id, processor_id=processor_id))
 
     async with transaction() as execute:
-        await operations.apply_tags(execute, entry_id, processor_id, uids_to_ids.values())
+        await operations.apply_tags(execute, entry_id, processor_id, list(uids_to_ids.values()))
         await operations.apply_tags_properties(execute, properties)
 
 
@@ -100,7 +78,9 @@ async def get_tags_info(tags_ids: Iterable[TagId]) -> dict[TagId, Tag]:  # noqa:
 
     tags_by_ids = await get_tags_by_ids(tags_ids)
 
-    info = {tag_id: Tag(id=tag_id, name=converters.verbose(tags_by_ids[tag_id])) for tag_id in tags_ids}
+    info = {
+        tag_id: Tag(id=tag_id, name=converters.verbose(tags_by_ids[tag_id])) for tag_id in tags_ids  # type: ignore
+    }
 
     # TODO: implement more complex merging
     for property in properties:
@@ -122,12 +102,8 @@ async def get_tags_info(tags_ids: Iterable[TagId]) -> dict[TagId, Tag]:  # noqa:
 
 @run_in_transaction
 async def remove_relations_for_entries(execute: ExecuteType, entries_ids: list[EntryId]) -> None:
-    await operations.remove_relations_for_entries(execute, entries_ids)
-
-
-@run_in_transaction
-async def tech_copy_relations(execute: ExecuteType, entry_from_id: EntryId, entry_to_id: EntryId) -> None:
-    await operations.tech_copy_relations(execute, entry_from_id, entry_to_id)
+    relation_ids = await operations.get_relations_for(execute, entry_ids=entries_ids)
+    await operations.remove_relations(execute, relation_ids)
 
 
 def _inplace_filter_out_entry_tags(
@@ -161,3 +137,37 @@ async def prepare_tags_for_entries(
     tag_mapping = await get_tags_by_ids(whole_tags)
 
     return entry_tag_ids, tag_mapping
+
+
+@run_in_transaction
+async def remove_orphaned_tags(execute: ExecuteType, chunk: int, protected_tags: list[TagId]) -> int:
+    orphaned_tags = await operations.get_orphaned_tags(execute, limit=chunk, protected_tags=protected_tags)
+
+    success = await operations.remove_tags(execute, orphaned_tags)
+
+    # Since we run the code in a transaction, we do nothing with relations here
+    # In case some relations will be added during the transaction, we encounter a foreign key violation
+    # => the transaction will be rolled back
+    # We could silence such failures, because cleaning orphaned tags is a periodic task
+
+    if success:
+        return len(orphaned_tags)
+
+    return 0
+
+
+@run_in_transaction
+async def copy_relations(execute: ExecuteType, processor_id: int, old_tag_id: TagId, new_tag_id: TagId) -> None:
+
+    relation_ids = await operations.get_relations_for(execute, tag_ids=[old_tag_id], processor_ids=[processor_id])
+
+    new_relation_ids = await operations.copy_relations_to_new_tag(execute, relation_ids, new_tag_id)
+
+    await operations.register_relations_processors(execute, new_relation_ids, processor_id)
+
+
+@run_in_transaction
+async def remove_relations_for_tags(execute: ExecuteType, tag_ids: list[TagId]) -> None:
+    relation_ids = await operations.get_relations_for(execute, tag_ids=tag_ids)
+
+    await operations.remove_relations(execute, relation_ids)
